require "option_parser"
require "ipc"
require "./colors"

verbosity = 1
service_name = "pong"

OptionParser.parse do |parser|
	parser.on "-s service_name", "--service-name service_name", "URI" do |optsn|
		service_name = optsn
	end

	parser.on "-v verbosity", "--verbosity verbosity", "Verbosity (0 = nothing is printed, 1 = only events, 2 = events and messages). Default: 1" do |optsn|
		verbosity = optsn.to_i
	end

	parser.on "-h", "--help", "Show this help" do
		puts parser
		exit 0
	end
end


IPC::Service.new (service_name) do |event|
	case event
	when IPC::Event::Timer
		if verbosity >= 1
			puts "#{CORANGE}IPC::Event::Timer#{CRESET}"
		end
	when IPC::Event::Connection
		if verbosity >= 1
			puts "#{CBLUE}IPC::Event::Connection#{CRESET}, client: #{event.connection.fd}"
		end
	when IPC::Event::Disconnection
		if verbosity >= 1
			puts "#{CBLUE}IPC::Event::Disconnection#{CRESET}, client: #{event.connection.fd}"
		end
	when IPC::Event::Message
		if verbosity >= 1
			puts "#{CGREEN}IPC::Event::Message#{CRESET}, client: #{event.connection.fd}"
			if verbosity == 2
				puts "#{CBLUE}message: #{event.message} #{CRESET}"
			end
		end
		event.connection.send event.message
	else
		if verbosity >= 1
			puts "#{CRED}Exception: message = #{event.message} #{CRESET}"
		end
	end
end
